// Hotkeys.cpp
// If you want to change what the hotkeys are, see the table at the bottom!

#include "config.h"

#include "Frame.h"
#include "Windowmanager.h"
#include "Desktop.h"

#include <efltk/Fl_Util.h>

#include "debug.h"

extern void show_tabmenu(int dir);

// Alt+Tab
static void NextWindow()
{ 
    show_tabmenu(1);
}

// Alt+Shift+Tab
static void PreviousWindow() {
    show_tabmenu(-1);
}

// Ctrl+Alt+Right
static void NextDesk()
{
    Desktop::next();
}

// Ctrl+Alt+Left
static void PreviousDesk()
{
    Desktop::prev();
}

// warning: this assummes it is bound to Fn key:
static void FKey_DeskNumber()
{
    Desktop::current(Desktop::desktop(Fl::event_key()-0xffbd));
}

static void FastRun()
{
    fl_start_child_process("erun &");
}

static void FindUtil()
{
    fl_start_child_process("efinder &");
}


////////////////////////////////////////////////////////////////
// I have to take some time to check it out
static struct {
    int key;
    void (*func)();
} keybindings[] = {

    {FL_ALT+FL_Tab,           NextWindow},
    {FL_ALT+FL_SHIFT+FL_Tab,  PreviousWindow},
    /*
    {FL_CTRL+FL_F(1),	FKey_DeskNumber},
    {FL_CTRL+FL_F(2),	FKey_DeskNumber},
    {FL_CTRL+FL_F(3),	FKey_DeskNumber},
    {FL_CTRL+FL_F(4),	FKey_DeskNumber},
    {FL_CTRL+FL_F(5),	FKey_DeskNumber},
    {FL_CTRL+FL_F(6),	FKey_DeskNumber},
    {FL_CTRL+FL_F(7),	FKey_DeskNumber},
    {FL_CTRL+FL_F(8),	FKey_DeskNumber},
    */
    // seem to be common to Linux window managers
    {FL_ALT+FL_F(1),	FKey_DeskNumber},
    {FL_ALT+FL_F(2),	FKey_DeskNumber},
    {FL_ALT+FL_F(3),	FKey_DeskNumber},
    {FL_ALT+FL_F(4),	FKey_DeskNumber},
    {FL_ALT+FL_F(5),	FKey_DeskNumber},
    {FL_ALT+FL_F(6),	FKey_DeskNumber},
    {FL_ALT+FL_F(7),	FKey_DeskNumber},
    {FL_ALT+FL_F(8),	FKey_DeskNumber},

    {FL_ALT+FL_CTRL+FL_Left,  PreviousDesk},
    {FL_ALT+FL_CTRL+FL_Right, NextDesk},

    {FL_ALT+FL_F(3),	FindUtil},
    {FL_ALT+FL_F(12),	FastRun},
    {0}
};

int Handle_Hotkey() {
    for (int i = 0; keybindings[i].key; i++) {
        if (Fl::test_shortcut(keybindings[i].key) ||
            (keybindings[i].key & 0xFFFF) == FL_Delete
            && Fl::event_key() == FL_BackSpace// fltk bug?
           ) {
            keybindings[i].func();
            return 1;
        }
    }
    return 0;
}

void Grab_Hotkeys()
{
    Window root_w = fl_xid(root);
    for (int i = 0; keybindings[i].key; i++) {
        int k = keybindings[i].key;
        int keycode = XKeysymToKeycode(fl_display, k & 0xFFFF);
        if(!keycode) continue;
        // Silly X!  we need to ignore caps lock & numlock keys by grabbing
        // all the combinations:
        XGrabKey(fl_display, keycode, k>>16,     root_w, 0, 1, 1);
        XGrabKey(fl_display, keycode, (k>>16)|2, root_w, 0, 1, 1); // CapsLock
        XGrabKey(fl_display, keycode, (k>>16)|16, root_w, 0, 1, 1); // NumLock
        XGrabKey(fl_display, keycode, (k>>16)|18, root_w, 0, 1, 1); // both
    }
}

